
import netgen.gui
from ngsolve import *
from netgen.geom2d import SplineGeometry


geo = SplineGeometry()
geo.AddRectangle( (-1, -1), (1, 1),
                 bcs = ("bottom", "right", "top", "left"))
mesh = Mesh( geo.GenerateMesh(maxh=0.25))
Draw(mesh)
fes = H1(mesh, order=3, dirichlet="bottom")

ngmesh = geo.GenerateMesh(maxh=0.15)
mesh = Mesh(ngmesh)


u = fes.TrialFunction()  # symbolic object
v = fes.TestFunction()   # symbolic object
gfu = GridFunction(fes)  # solution

gfu.Set(283.15,BND)
Draw(gfu)


K = 1 #thermal conductivity
sources = 1 #soure term

#heat equation
a = BilinearForm(fes, symmetric=False)
a += SymbolicBFI(K*grad(u)*grad(v)) #dx causes error: 'not defined'
a.Assemble()

m = BilinearForm(fes, symmetric=False)
m +=SymbolicBFI(u*v)
m.Assemble()

f = LinearForm(fes)
f += SymbolicLFI(sources*v)
f.Assemble()

rhs = f.vec.CreateVector()
rhs.data = f.vec - a.mat * gfu.vec

#%%

#Implicit Euler scheme
dt = 0.001
t_end = 1
gfu.Set(293.15) #set intial condition

mstar = m.mat.CreateMatrix()
mstar.AsVector().data = m.mat.AsVector() + dt * a.mat.AsVector()
invmstar = mstar.Inverse(freedofs=fes.FreeDofs())

res = gfu.vec.CreateVector()
 
t = 0.0
while t <= t_end:
    res.data = dt * rhs - dt * a.mat * gfu.vec
    gfu.vec.data += invmstar * res
    t = t+dt
    Redraw(blocking=True)
    


